import sys
import time
import random
import os 
import math
import pygame
import pygame.mixer
from terrain import Terrain
from steamjet import SteamJet
from traincar import TrainCar,Locomotive 
from tools import *

class Game:
   def __init__(self,size,terrain_img,game_over):
      self.terrain = Terrain(size,terrain_img)
      self.cars=[]
      self.couplings=[]
      self.gravity_const=10
      self.locomotive=None
      self.game_over = game_over
      # fully compressed size,compressed to start moving size,
      # uncompressed to start pulling size, fully uncompressed size,
      # compressive spring constant (kg/s**2)
      # expansion spring constant (kg/s**2)
      self.couple_consts=(0.05,0.2,0.4,0.55)
      self.kc,self.ke = 5e7,0.625e7
      self.dc,self.de = 4e5,4e5
      self.dt=0.005
      self.score=0

   # Dave Manthey (www.orbitals.com) did all the brain stuff on this one      
   def tick(self):
      for i in range(len(self.cars)):
         if i > 0:
            cpl0 = self.distance_and_force_on_couple(self.cars[i],self.cars[i-1],False)
            if cpl0[0] > self.couple_consts[3]: cpl0 = (cpl0[0],0.0)  # don't apply force of coupling if it is broken
         else:
            cpl0 = (0.0,0.0)
         if i < len(self.couplings):
            cpl1 = self.distance_and_force_on_couple(self.cars[i+1],self.cars[i],True)
            if cpl1[0] > self.couple_consts[3]: cpl1 = (cpl1[0],0.0) # as above
         else:
            cpl1 = (0.0,0.0)
         if i < len(self.couplings):
            self.couplings[i]=cpl1
         self.cars[i].cpl=(cpl0,cpl1)
         tf = self.cars[i].total_force(cpl0[1],cpl1[1])
         vp = self.cars[i].vel + self.dt*tf/self.cars[i].mass
         try:
            xp = self.cars[i].x + self.dt*self.cars[i].vel*math.cos(self.cars[i].inc)+0.5*self.dt*((tf/self.cars[i].mass)**2)
         except OverflowError:
            print "Overflow on xp calc of car %i"%i
            sys.exit()
         self.cars[i].vel = vp 
         self.cars[i].x = xp
         #print "Car %i: tf:%s vp:%s xp:%s" % (i,str(tf),str(vp),str(xp))
         #if i < len(self.couplings): print "Coupling %i: length:%f force:%s" % (i,self.couplings[i][0],str(self.couplings[i][1]))
      for car in self.cars:
         car.tick(self.terrain)
      self.score=0
      for i in range(len(self.cars)):
         if self.cars[i].x > self.terrain.end:
            self.score+=1
         if i < len(self.couplings) and self.couplings[i][0] > self.couple_consts[3] or self.couplings[i][1] < self.couple_consts[0]:
            break
      if i!=0 and i == self.score:
         self.game_over()
              
   def distance_and_force_on_couple(self,car0,car1,side):
      dcpl=(car1.x - car0.x - (0.5*car0.length*math.cos(car0.inc))-(0.5*car1.length*math.cos(car1.inc))) / \
            math.cos( 0.5*(car1.inc+car0.inc))
      if side: side1,side2=1,-1
      else: side2,side1=1,-1

      #print "%f - %f - %f"%(car0.x,dcpl,car1.x)
      cc=self.couple_consts 
      if dcpl < cc[1]:
         fk = side1 * self.kc*(dcpl-cc[1])/(cc[0]-cc[1]) # -kx where x is the displacement
         fd = self.dc*(car0.vel-car1.vel) 
         return (dcpl,fd+fk)
      elif dcpl <= cc[2]:
         return (dcpl,0.0)
      elif cc[2] < dcpl:
         fk = side2 * self.ke*(dcpl-cc[2])/(cc[3]-cc[1])
         fd = self.de*(car1.vel-car0.vel) # -bv where b is the dampening constant and v is the difference in velocity
         return (dcpl,fd+fk) 
      else:
         return (dcpl,0)
   
   def render(self,screen):
      self.terrain.render(screen,self.locomotive.x)
      for car in self.cars:
         car.render(screen,self.terrain) 

   def initialize(self,tcar_imgs,loco_img,loco_steamoffset):
      self.cars=[]
      self.locomotive=[]
      self.couplings=[]
      self.terrain.initialize()
      self.terrain.game_ratio = loco_img.get_width()/14.0 # real to pixel ratio
      self.terrain.end = self.terrain.background.get_width()/self.terrain.game_ratio
      cpl_init = self.couple_consts[1] + (self.couple_consts[2]-self.couple_consts[1])/2 # initial coupling size
      self.cars.append(Locomotive(100.0,14,125000,loco_img,8.0,loco_steamoffset))
      self.locomotive=self.cars[0]
      print "Added locomotive at (%f), of length %i"%(self.locomotive.x,self.locomotive.length)
      last=self.locomotive.x-7.0-cpl_init
      for i in range(1,9):
         tcar_img=tcar_imgs[random.randint(0,len(tcar_imgs)-1)]
         self.cars.append(TrainCar(last-6.0,12,125000,tcar_img))
         last -= 12.0 + cpl_init
         print "Added car %i at (%f), of length %i"%(i,self.cars[-1].x,self.cars[-1].length)
         self.couplings.append((cpl_init,0.0))
      # initialize the cars
      last=None
      for car in self.cars:
         car.tick(self.terrain)
         if last != None:
            print "%f -> %f: %f" % (last,car.x+car.length/2,car.x+car.length/2-last)
         last = car.x-car.length/2


