import sys
import time
import random
import os 
import math
import pygame
import pygame.mixer
from terrain import Terrain
from steamjet import SteamJet
from traincar import TrainCar,Locomotive 
from menu import Menu
from tools import *
from game import Game

TICK = pygame.USEREVENT + 1 


def render():
   screen.blit(bg_img,(game.terrain.spos,0,screen.get_width(),screen.get_height()))
   game.render(screen)
   fxsurf.fill((0,0,0,0))
   hudsurf.fill((0,0,0,0))
   loco_steamjet.render(fxsurf,game.terrain.spos)
   screen.blit(fxsurf,(0,0))

   if mode == 0:
      menu.render(screen)
   elif mode == 1:
      render_hud(hudsurf)
      screen.blit(hudsurf,(0,0))     
   elif mode == 2:
      render_help(screen)
   elif mode == 3:
      render_score(screen)

def game_over():
   set_mode(3)

def tick():
   if play:
      for i in range(10):
         if not play: break
         game.tick()  
   for i in range(int(2+abs(game.locomotive.throttle)*5)):
      src,dir = game.locomotive.get_steamsource(fxsurf,game.terrain)
      loco_steamjet.add_particle(src,dir)
   loco_steamjet.tick()

def render_help(screen):
   help_node=help_path[helpstep]
   screen.blit(help_node[1],(10,10))
   screen.blit(help_bubble,(15+help_node[1].get_width(),10))
   x,y=help_node[1].get_width()+15+70,15
   for s in help_node[0].split('\n'):
      tsurf=helpfont.render(s,True,(100,100,255))
      screen.blit(tsurf,(x,y))
      y+=tsurf.get_height()

def render_score(screen):
   if game.score < len(game.cars):
      txt="You lost %i cars. Sorry, thats not good enough.\nBut I have faith in you, why don't you try again,\njust go easy on that throttle."
      icon=help_icon2
   else:
      txt="Congratulations! You made it. Good Job!\nWe need more engineers like you in our Railroad.\nFeel free to drive another train any time!"
      icon=help_icon1
   screen.blit(icon,(10,10))
   screen.blit(help_bubble,(15+icon.get_width(),10))
   x,y=icon.get_width()+15+70,15
   for s in icon.split('\n'):
      tsurf=helpfont.render(s,True,(100,100,255))
      screen.blit(tsurf,(x,y))
      y+=tsurf.get_height()

def render_hud(screen):
   #print "Rendering HUD"
   y=5
   i=0
   #for c in game.cars:
   #   tsurf = f1.render("car %i x:%f tf:%f v:%f inc:%f Couplings(%f,%s) (%f,%s)"%\
      #(i,c.x,c.tf,c.vel,c.inc,c.cpl[0][0],str(c.cpl[0][1]),c.cpl[1][0],str(c.cpl[1][1])),True,(255,200,200))
   #   screen.blit(tsurf,(5,y)) 
   #   y+=tsurf.get_height()+2
   #   i+=1
   screen.blit(hud_img,(0,0))
   #tsurf = f1.render("Throttle: %02f"%game.locomotive.throttle,True,(200,200,0))
   #screen.blit(tsurf,(screen.get_width()-tsurf.get_width(),5+tsurf.get_height()))
   if play: 
      pygame.draw.circle(screen,(0,255,0),(758,76),15)
   else:
      pygame.draw.circle(screen,(0,255,0),(758.36),11)
   #   pcolor=(0,255,0)
   #   ptxt="Playing"
   #else:
   #   pcolor=(255,0,0)
   #   ptxt="Stopped"
   #tsurf = f1.render(ptxt,True,pcolor)
   #screen.blit(tsurf,(screen.get_width()-tsurf.get_width()-5,15+tsurf.get_height()))
   
   i=0
   cplw=coupling_gauge_img.get_width()+10
   cplh=coupling_gauge_img.get_height()/2
   cpl_init = (game.couple_consts[2]+game.couple_consts[1])/2 # initial coupling size
   for c in game.couplings:
      screen.blit(coupling_gauge_img,(i*cplw+96,515))
      tsurf=None
      if c[0] > cpl_init and c[0] <= game.couple_consts[3]: amt = (c[0]-cpl_init)/(game.couple_consts[3]-cpl_init)
      elif c[0] <= cpl_init and c[0] >= game.couple_consts[0]: amt = (cpl_init-c[0])/(cpl_init-game.couple_consts[0])
      elif c[0] > game.couple_consts[3]:
         amt = 1.0
         tsurf = f1.render("Break",True,(255,0,0))
      elif c[0] < game.couple_consts[0]:
         amt=0.0
         tsurf = f1.render("Derail",True,(255,0,0))
      needle = rotate_vec(((cplw-10)/2+3,0),math.radians(235-(270*amt)))
      pygame.draw.line(screen,(0,0,0),((i*cplw)+96+(cplw-10)/2,515+cplh+3),(int(needle[0])+(i*cplw)+96+(cplw-10)/2,515+cplh+int(needle[1])+3),2)
      if tsurf!= None: screen.blit(tsurf,((i*cplw)+96,515+cplh*2+5))
      i+=1
   # Gauge centers: Throttle (69,64), Speed (197,64) 
   needle = rotate_vec((35,0),math.radians(235-(270*game.locomotive.throttle)))
   pygame.draw.line(screen,(0,0,0),(69,64),(int(needle[0])+69,64+int(needle[1])),2) 
   needle = rotate_vec((35,0),math.radians(235-(270*(game.locomotive.vel/20.0)))) # Testing shows20 to be a decent max velocity..
   pygame.draw.line(screen,(0,0,0),(197,64),(int(needle[0])+197,64+int(needle[1])),2) 

def checkKeys():
   keys = pygame.key.get_pressed()
   global mode
   global last_helpkey
   global helpstep
   if mode==3:
      for key in keys:
         if key:
            set_mode(0)
            return
   if keys[pygame.K_F12]:
      pygame.image.save(screen,"screenshot%i.bmp"%time.time())
   if mode==0:
      menu.handle_keys(keys)
   elif mode==2:
      if keys[pygame.K_SPACE] and last_helpkey + 0.02 < time.time():
         helpstep +=1
         if helpstep >= len(help_path):
            set_mode(0) 
            helpstep=0
         last_helpkey=time.time()
   else:
      if keys[pygame.K_SPACE]:
         pass
         #pygame.image.save(screen,"screenshot%i.bmp"%time.time())
         #play = False 
         #game.initialize(tcar_imgs,loco_img,loco_steamoffset)  
      if keys[pygame.K_ESCAPE]:
         set_mode(0)
      if keys[pygame.K_RIGHT]:
         game.locomotive.throttle += 0.01
         if game.locomotive.throttle > 1.0: # max out at 100% of course
            game.locomotive.throttle = 1.0
      elif keys[pygame.K_LEFT]:
         game.locomotive.throttle -= 0.01
         if game.locomotive.throttle < -1.0: # max out at 100% of course
            game.locomotive.throttle = -1.0

def set_mode(new_mode):
   global play
   global mode
   if new_mode==0:
      play=False
      mode=new_mode
   elif new_mode==1:
      play=True
      mode=new_mode
   elif new_mode==2:
      play=False
      mode=new_mode

def menu_new_game():
   game.initialize(tcar_imgs,loco_img,loco_steamoffset)
   set_mode(1)

def menu_help():
   set_mode(2)

def menu_settings():
   pass

def menu_credits():
   pass

def menu_exit():
   sys.exit()

# The Game
pygame.init()
size = width, height = 800, 600
screen = pygame.display.set_mode(size)
play=False
mb_down=False
mode=0 # 0 for menu, 1 for Game, 2 for help

#load the sprites
tcar_imgs = [pygame.image.load("content/traincar1.png").convert_alpha(),pygame.image.load("content/traincar2.png").convert_alpha(),\
    pygame.image.load("content/traincar3.png").convert_alpha()]
loco_img = pygame.image.load("content/locomotive.png").convert_alpha()
loco_steamoffset=(12,-25) # spot where steam comes out!
title_img = pygame.image.load("content/title_img.png").convert_alpha()
help_icon1 = pygame.image.load("content/engineer1.png").convert_alpha()
help_icon2 = pygame.image.load("content/engineer2.png").convert_alpha()
helpfont = pygame.font.SysFont("Arial",18)
help_bubble = pygame.image.load("content/speech_bubble.png").convert_alpha()
terrain_img=pygame.image.load("content/terrain.png").convert_alpha()
hud_img = pygame.image.load("content/hud.png").convert_alpha()
coupling_gauge_img = pygame.image.load("content/coupling_gauge.png").convert_alpha()
bg_img = pygame.image.load("content/bg.png").convert()

help_path=[\
("""Welcome to Nikolaj's Train Game. In this game you will be
the engineer of a locomotive. Your challenge is to successfully
drive the train over a series of hills without breaking it apart.
""",help_icon1),\
("""Driving trains isn't so easy. The couplings between the trains
can only take so much. Be gentle on the throttle and brakes and
pay attention to your couplings, shown at the bottom!
""",help_icon2),\
("""To control the train, use the right/left arrow keys to control
the throttle, and the space bar to apply the brakes on all the
trains. Careful, braking is not instantaneous on trains!
""",help_icon2),\
("""But you look like a competent engineer, and I am sure you
have what it takes! Right then, no more chitchat, we have goods
to deliver and an economy to support. All Aboard!
""",help_icon1)\
]
helpstep=0
last_helpkey=time.time()

#create and initialize the game object
game = Game(size,terrain_img,game_over)
game.initialize(tcar_imgs,loco_img,loco_steamoffset)
loco_steamjet = SteamJet()

# create the game surfaces
fxsurf = pygame.Surface(size,pygame.SRCALPHA,32)
hudsurf = pygame.Surface(size,pygame.SRCALPHA,32)
f1 = pygame.font.SysFont("Arial",12)
f2 = pygame.font.SysFont("Arial",10)
menufont = pygame.font.SysFont("Arial",20)

#Create the menu
opt_map = [("New Game",menu_new_game),("Settings",menu_settings),("Help",menu_help),("Credits",menu_credits),("Exit",menu_exit)]
menu = Menu(title_img,menufont,opt_map)

screen.fill((0,0,0))
pygame.display.update()
pygame.time.set_timer(TICK,100)

pygame.mixer.music.load("content/trainsong.mp3")
pygame.mixer.music.play(-1)

# and loop
while 1:
   event = pygame.event.wait()
   if event.type == pygame.QUIT: 
      sys.exit()
   elif event.type == pygame.MOUSEBUTTONDOWN:
      mb_down=True
   elif event.type == pygame.MOUSEBUTTONUP:
      print "Toggling Game"
      play=not play
      mb_down=False
   elif event.type == TICK:
      checkKeys() 
      #screen.fill((0,0,0))
      tick()
      render()
      pygame.display.flip()

