import sys
import time
import random
import os 
import math
import pygame
from steamjet import SteamJet
from tools import *

grav_const=9.8

class TrainCar:
   def __init__(self,x,length,mass,img):
      self.mass=mass
      self.x=x # in real units (meters)
      self.co_stat=0.02 #Coefficient of static friction
      self.co_roll=0.018 #Coefficient of rolling friction
      self.length=length # in real units (meters)
      self.img = img
      self.img_w = img.get_width()/2
      self.img_h = img.get_height()
      self.normalf=0
      self.ffric=0
      self.inc=0
      self.cpl=((0,0),(0,0))
      self.vel=0
      self.tf=0.0
      self.f1=pygame.font.SysFont("Arial",12)
      self.cdp = 0.5
      self.drag = 0.001
     
   def tick(self,terrain):
      self.inc=terrain.incline(self.x-self.length/2,self.x+self.length/2)
      # calculate forces
      self.normalf = self.mass * grav_const * math.cos(self.inc)
      if self.vel==0: self.ffric=self.normalf * self.co_stat
      else: self.ffric=self.normalf * self.co_roll

   def total_force(self,fcpl0,fcpl1):
      #print "Calculating Car total Force (fric: %f)"%self.ffric
      Fd = self.normalf*self.cdp*(self.vel**2)*self.drag
      if self.vel > 0: Fd *= -1
      if abs(fcpl0+fcpl1) < self.ffric: self.tf= 0.0
      elif fcpl0+fcpl1<0: self.tf= fcpl0+fcpl1+self.ffric+Fd
      elif fcpl0+fcpl1>0: self.tf= fcpl0+fcpl1-self.ffric+Fd
      else: self.tf= 0.0
      return self.tf

   def brake(self,delta):
      self.co_roll += delta 
   
   def render(self,surf,terrain):
      x = terrain.x_to_screen(self.x)
      img_y_adj = rotate_vec((self.img_w,0),self.inc)[1] #adjust for rotated traincar img
      if self.inc > 0: img_y_adj *= 0-1
      tc = pygame.transform.rotate(self.img,math.degrees(self.inc))
      if x-self.img_w >= terrain.screen_width or x+self.img_w < 0:
         return # car is off the screen
      surf.blit(tc,(x-self.img_w,surf.get_height()-terrain.get_screen_y(x)-self.img_h-img_y_adj))
      y=surf.get_height()-terrain.get_screen_y(x)-self.img_h/2
      #pygame.draw.line(surf,(255,255,255),(x,y),(x+self.vel,y))
      #tsurf=self.f1.render("x:%0.1f inc:%0.1f v:%0.1f"%(self.x,math.degrees(self.inc),self.vel),True,(200,200,255))
      #surf.blit(tsurf,(x-self.img_w/2,y-self.img_h))
      
class Locomotive(TrainCar):
   def __init__(self,x,length,mass,img,power,steam_offset):
      TrainCar.__init__(self,x,length,mass,img)
      self.power=power
      self.throttle=0.0 # between -1 and 1
      self.engine_force=0.0 # force imparted by engine
      self.offs = steam_offset # where to draw the steam relative to the center of the image in screen coords
      self.drag = 0.001
      self.cdp = 2 # wind friction for drag

   def tick(self,terrain):
      TrainCar.tick(self,terrain)
      self.engine_force=self.ffric * self.throttle * self.power

   def total_force(self,fcpl0,fcpl1):
      Fs = fcpl0+fcpl1+self.engine_force
      try:
         Fd = self.normalf*self.cdp*(self.vel**2)*self.drag
      except OverflowError:
         print "Oops total force overflow!"
         sys.exit()
      if self.vel >0: Fs -= Fd 
      elif self.vel < 0: Fs += Fd
      if abs(Fs) < self.ffric: self.tf= 0.0
      elif Fs<0: self.tf= Fs + self.ffric
      elif Fs>0: self.tf= Fs -self.ffric
      else: self.tf= 0.0
      #print "Calculating Locomotive Total Force (%f Fs,%f engine force,fric %f) => %f"%(Fs,self.engine_force,self.ffric,self.tf)
      return self.tf
      
   def get_steamsource(self,surf,terrain):
      x=terrain.x_to_screen(self.x)
      src= (x+self.offs[0],surf.get_height()-terrain.get_screen_y(x)+self.offs[1])
      dir = rotate_vec((-int(0.5*self.vel),-random.randint(15,30)),self.inc)
      return (src[0]+terrain.spos,src[1]),(int(dir[0]),int((-self.throttle*10)+dir[1]))#we actually need to offset this to clipped screen coords.. what a mess!

